/***************************************************************************************
 * @file     board.h
 * @version  V1.0
 * @date     18 January 2018
 *
 * @note
 * VORAGO Technologies
 *
 * @note
 * Copyright (c) 2013-2019 VORAGO Technologies.
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT.
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/
 
#ifndef __BOARD_H
#define __BOARD_H

/*****************************************************************************/ 
/* Include files                                                             */ 
/*****************************************************************************/

#include <stdint.h>
#include <stdbool.h>
#include "va108xx.h"

/*****************************************************************************/ 
/* Global pre-processor symbols/macros ('#define')                           */ 
/*****************************************************************************/

/** Version */
#define SOFTWARE_VERSION_STR  "2019_06_13_v1"
#define SOFTWARE_VERSION (0x00000613) // 0.6.13

// assert enable/disable (comment out to disable)
//#define USE_ASSERT

// debug print enable/disable (comment out to disable)
#define DEBUG_PRINTS

// if defined, tells the va108xx_debug module to not use the HAL UART driver
#define DEBUG_NO_HAL

/* Target Board - Define One of the fololwing: */
#define BRD_RH_OBC1
//#define BRD_REB1
//#define BRD_SIP_EVK

/* Target board multiple definition check */
#ifdef BRD_RH_OBC1
#undef BRD_REB1
#undef BRD_SIP_EVK
#endif
#ifdef BRD_REB1
#undef BRD_SIP_EVK
#endif

//missing from our design environment
#define STAT_SEL_PWMA      (0x3)
#define STAT_SEL_PWMB      (0x4)
#define FUNSEL0 0
#define FUNSEL1 1
#define FUNSEL2 2
#define FUNSEL3 3

#undef VOR_PORTA
#undef VOR_PORTB
#define VOR_PORTA           (VOR_GPIO->BANK[0])
#define VOR_PORTB           (VOR_GPIO->BANK[1])

#define VOR_UART0           (VOR_UARTA)
#define VOR_UART1           (VOR_UARTB)

#define VOR_SPI0            (VOR_SPI->BANK[0])
#define VOR_SPI1            (VOR_SPI->BANK[1])

/* System Clock in Hz. */
#define SYSCLK	(50000000)
#define configUART_BAUDRATE (115200)

/* CRC calculation length timer */
#define VOR_TIM_CRC_INDEX   (18)
#define VOR_TIM_CRC         (VOR_TIM->BANK[VOR_TIM_CRC_INDEX])

/* Timer that generates second tick. */
#define VOR_TIM_SEC_INDEX   (19)
#define VOR_TIM_SEC         (VOR_TIM->BANK[VOR_TIM_SEC_INDEX])
#define VOR_IRQ_SEC_INDEX 		((IRQn_Type)27)

/* Millisecond timer */
#define VOR_TIM_MS_INDEX   (20)
#define VOR_TIM_MS         (VOR_TIM->BANK[VOR_TIM_MS_INDEX])
#define VOR_IRQ_TIM_MS_INDEX 		((IRQn_Type)28)

/** RH-OBC1 pin assignments */
#ifdef BRD_RH_OBC1
#define WD_PORT              PORTA
#define WD_BANK              VOR_GPIO->BANK[0]
#define WD_PIN               (24)

#define LED_PORT             PORTB
#define LED_BANK             VOR_GPIO->BANK[1]
#define LED_PIN              (7)

#define UARTA_TX_PORT        PORTA
#define UARTA_TX_BANK        VOR_GPIO->BANK[0]
#define UARTA_TX_PIN         (17)
#define UARTA_TX_FUNSEL      (FUNSEL3)

#define UARTA_RX_PORT        PORTA
#define UARTA_RX_BANK        VOR_GPIO->BANK[0]
#define UARTA_RX_PIN         (16)
#define UARTA_RX_FUNSEL      (FUNSEL3)
#endif

/** REB1 pin assignments */
#ifdef BRD_REB1
#define UARTA_TX_PORT        PORTA
#define UARTA_TX_BANK        VOR_GPIO->BANK[0]
#define UARTA_TX_PIN         (31)
#define UARTA_TX_FUNSEL      (FUNSEL3)
#define UARTA_BAUDRATE       (configUART_BAUDRATE)

#define UARTA_RX_PORT        PORTA
#define UARTA_RX_BANK        VOR_GPIO->BANK[0]
#define UARTA_RX_PIN         (30)
#define UARTA_RX_FUNSEL      (FUNSEL3)

#define MAX11619_CS_N_PORT   PORTA
#define MAX11619_CS_N_BANK   VOR_GPIO->BANK[0]
#define MAX11619_CS_N_PIN    (17)

#define LED_D2_PORT          PORTA
#define LED_D2_BANK          VOR_GPIO->BANK[0]
#define LED_D2_PIN           (10)

#define LED_D3_PORT          PORTA
#define LED_D3_BANK          VOR_GPIO->BANK[0]
#define LED_D3_PIN           (7)

#define LED_D4_PORT          PORTA
#define LED_D4_BANK          VOR_GPIO->BANK[0]
#define LED_D4_PIN           (6)
#endif

/** SIP-EVK pin assignments */
#ifdef BRD_SIP_EVK
// DS4 LED
#define LED_DS4_PORT         PORTA
#define LED_DS4_BANK         VOR_GPIO->BANK[0]
#define LED_DS4_PIN          (21)

// DS5 LED
#define LED_DS5_PORT         PORTA
#define LED_DS5_BANK         VOR_GPIO->BANK[0]
#define LED_DS5_PIN          (22)
#define LED_DS5_TIM          (0)

// SW2 switch
#define SW2_PORT             PORTA
#define SW2_BANK             VOR_GPIO->BANK[0]
#define SW2_PIN              (15)

// UARTA (UART0)
#define UARTA_RX_PORT        PORTA
#define UARTA_RX_BANK        VOR_GPIO->BANK[0]
#define UARTA_RX_PIN         (16)
#define UARTA_RX_FUNSEL      (FUNSEL3)

#define UARTA_TX_PORT        PORTA
#define UARTA_TX_BANK        VOR_GPIO->BANK[0]
#define UARTA_TX_PIN         (17)
#define UARTA_TX_FUNSEL      (FUNSEL3)

#define UARTA_BAUDRATE       (115200)


// UARTB (UART1)
#define UARTB_RX_PORT        PORTA
#define UARTB_RX_BANK        VOR_GPIO->BANK[0]
#define UARTB_RX_PIN         (26)
#define UARTB_RX_FUNSEL      (FUNSEL3)

#define UARTB_TX_PORT        PORTA
#define UARTB_TX_BANK        VOR_GPIO->BANK[0]
#define UARTB_TX_PIN         (27)
#define UARTB_TX_FUNSEL      (FUNSEL3)

#define UARTB_BAUDRATE       (115200)
#define VOR_UART1_IRQ_INDEX	 ((IRQn_Type)4)

// SPIA (SPI0)
#define SPIA_SCK_PORT        PORTA
#define SPIA_SCK_BANK        VOR_GPIO->BANK[0]
#define SPIA_SCK_PIN         (31)
#define SPIA_SCK_FUNSEL      (FUNSEL1)

#define SPIA_MOSI_PORT       PORTA
#define SPIA_MOSI_BANK       VOR_GPIO->BANK[0]
#define SPIA_MOSI_PIN        (30)
#define SPIA_MOSI_FUNSEL     (FUNSEL1)

#define SPIA_MISO_PORT       PORTA
#define SPIA_MISO_BANK       VOR_GPIO->BANK[0]
#define SPIA_MISO_PIN        (29)
#define SPIA_MISO_FUNSEL     (FUNSEL1)

// PMOD SPI CSn Pin: PA23 FUN1 -> SSA5
#define PMOD_SPIA_CS_PORT    PORTA
#define PMOD_SPIA_CS_BANK    VOR_GPIO->BANK[0]
#define PMOD_SPIA_CS_PIN     (23)
#define PMOD_SPIA_CS_NUM     (5)
#define PMOD_SPIA_CS_FUNSEL  (FUNSEL1)

// ROM1 CSn Pin: PA28 FUN1 -> SSA0
#define ROM1_SPIA_CS_PORT    PORTA
#define ROM1_SPIA_CS_BANK    VOR_GPIO->BANK[0]
#define ROM1_SPIA_CS_PIN     (28)
#define ROM1_SPIA_CS_NUM     (0)
#define ROM1_SPIA_CS_FUNSEL  (FUNSEL1)

// ROM1 (external) WPn pin
#define ROM1_WP_N_PORT       PORTB
#define ROM1_WP_N_BANK       VOR_GPIO->BANK[1]
#define ROM1_WP_N_PIN        (23)
#define ROM1_WP_N_FUNSEL     (FUNSEL0)
#endif

// SPI 
#define SPI0_NUM             (0)
#define SPI1_NUM             (1)
#define SPI2_NUM             (2)

// IRQns
#define VOR_UART0_IRQ_INDEX		((IRQn_Type)3)
#define VOR_UART1_IRQ_INDEX		((IRQn_Type)4)

#define SPI0_RX_IRQn		     ((IRQn_Type)6)
#define SPI1_RX_IRQn		     ((IRQn_Type)7)
#define SPI2_RX_IRQn		     ((IRQn_Type)8)

/*****************************************************************************/
/* Variable declarations                                                     */
/*****************************************************************************/

/* Alias for GPIO Ports. */
extern VOR_GPIO_PERIPHERAL_Type *PORTA;
extern VOR_GPIO_PERIPHERAL_Type *PORTB;

/*****************************************************************************/ 
/* Global function prototypes ('extern', definition in C source)             */ 
/*****************************************************************************/

#if defined ( __CC_ARM )
/* watchdog feed */
__INLINE void WDFeed(){
#ifdef BRD_RH_OBC1
  WD_BANK.TOGOUT=1UL<<WD_PIN;
#endif
}
#elif defined ( __ICCARM__ )
extern __INLINE void VOR_GPIO_PinMux(VOR_GPIO_PERIPHERAL_Type *port, uint32_t pin, uint32_t funsel);
extern __INLINE void WDFeed(void);
#endif

/*****************************************************************************/ 
/* End of file                                                               */ 
/*****************************************************************************/
#endif // __BOARD_H

