/***************************************************************************************
 * @file     main.c
 * @version  V1.0
 * @date     31 March 2018
 *
 * @note
 * VORAGO Technologies
 *
 * @note
 * Copyright (c) 2013-2016 VORAGO Technologies.
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT.
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/
 
/*****************************************************************************/ 
/* Include files                                                             */ 
/*****************************************************************************/

/* System */
#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>
#include "va108xx.h"
#include "board.h"

/* Bootloader components */
#include "uart.h"
#include "bootloader.h"

/** VA108xx hal includes */ 
#include "va108xx_debug.h"

/*****************************************************************************/ 
/* Global variable definitions (declared in header file with 'extern')       */ 
/*****************************************************************************/

/* Alias for GPIO Ports. */
VOR_GPIO_PERIPHERAL_Type *PORTA = &(VOR_GPIO->BANK[0]);
VOR_GPIO_PERIPHERAL_Type *PORTB = &(VOR_GPIO->BANK[1]);

volatile uint32_t u32SecondsCounter=0;
volatile uint32_t u32SecToggle=0;
volatile uint32_t u32MsCounter=0;
volatile uint32_t u32MsToggle=0;

/*****************************************************************************/ 
/* Function implementation - global ('extern') and local ('static')          */ 
/*****************************************************************************/

__INLINE uint32_t NVIC_checkIRQ(uint32_t IRQn)
{
  return (NVIC->ISPR[0U] & (1UL << IRQn));
}

__INLINE void NVIC_clrPendingIRQ(uint32_t IRQn)
{
  NVIC->ICPR[0U] = (1UL << IRQn);
}

static void VOR_GPIOA_PinMux(uint32_t pin, uint32_t funsel) {
		VOR_IOCONFIG->PORTA[pin] = (VOR_IOCONFIG->PORTA[pin] & ~((0x3)<<IOCONFIG_PORTA_FUNSEL_Pos))	| ((funsel)<<IOCONFIG_PORTA_FUNSEL_Pos);
}
static void VOR_GPIOB_PinMux(uint32_t pin, uint32_t funsel) {
		VOR_IOCONFIG->PORTB[pin] = (VOR_IOCONFIG->PORTB[pin] & ~((0x3)<<IOCONFIG_PORTB_FUNSEL_Pos))	| ((funsel)<<IOCONFIG_PORTB_FUNSEL_Pos);
} 

__INLINE void VOR_GPIO_PinMux(VOR_GPIO_PERIPHERAL_Type *port, uint32_t pin, uint32_t funsel) {
  if(port==PORTA)
	  VOR_GPIOA_PinMux(pin,funsel);
	else
		VOR_GPIOB_PinMux(pin,funsel);
}

/* watchdog feed */
#if defined ( __ICCARM__ )
__INLINE void WDFeed(){
#ifdef BRD_RH_OBC1
  WD_BANK.TOGOUT=1UL<<WD_PIN;
#endif
}
#endif

void SecTimerInit(void)
{ 
    /* Configure Timer to generate interupt every second, for internal
       timekeeping. */
	  VOR_SYSCONFIG->TIM_CLK_ENABLE |= (1UL << VOR_TIM_SEC_INDEX);
    VOR_TIM_SEC.CTRL = (TIM_PERIPHERAL_CTRL_IRQ_ENB_Msk |
                        (STAT_SEL_PWMA << TIM_PERIPHERAL_CTRL_STATUS_SEL_Pos));
    VOR_TIM_SEC.RST_VALUE = SYSCLK;  
    VOR_TIM_SEC.CNT_VALUE = 0;
    
    VOR_IRQSEL->TIM[VOR_TIM_SEC_INDEX] = VOR_IRQ_SEC_INDEX;
    NVIC_SetPriority(VOR_IRQ_SEC_INDEX, 1);
    NVIC_EnableIRQ(VOR_IRQ_SEC_INDEX);
    
    VOR_TIM_SEC.ENABLE = 1;
}

void MsTimerInit(void)
{ 	
		/* Configure Timer to generate interupt every millisecond, for internal
       timekeeping. */
	  VOR_SYSCONFIG->TIM_CLK_ENABLE |= (1UL << VOR_TIM_MS_INDEX);
    VOR_TIM_MS.CTRL = (TIM_PERIPHERAL_CTRL_IRQ_ENB_Msk |
                        (STAT_SEL_PWMA << TIM_PERIPHERAL_CTRL_STATUS_SEL_Pos));
    VOR_TIM_MS.RST_VALUE = SYSCLK/1000;//1ms
    VOR_TIM_MS.CNT_VALUE = 0;
    
    VOR_IRQSEL->TIM[VOR_TIM_MS_INDEX] = VOR_IRQ_TIM_MS_INDEX;
    NVIC_SetPriority(VOR_IRQ_TIM_MS_INDEX, 1);
    NVIC_EnableIRQ(VOR_IRQ_TIM_MS_INDEX);
    
    VOR_TIM_MS.ENABLE = 1;
}


/*******************************************************************************
 **
 ** @brief  Enables weak pulldowns on all pins.
 ** Sets WDT direction
 **
 ******************************************************************************/
void ConfigureIO(void)
{
  uint32_t pin;
 
  // Enable pulldowns on all pins
  for(pin = 0; pin < 32; pin++)
  {
    VOR_IOCONFIG->PORTA[pin] |= IOCONFIG_PORTA_PEN_Msk;
    VOR_IOCONFIG->PORTB[pin] |= IOCONFIG_PORTB_PEN_Msk;
  }
  PORTA->DIR = 0;
  PORTB->DIR = 0;
  
#ifdef BRD_RH_OBC1
  // RH-OBC1 board
  
  //Create default bus condition if FUNSEL moves
  UARTA_TX_BANK.DIR   |=1UL<<UARTA_TX_PIN;
  UARTA_TX_BANK.SETOUT =1UL<<UARTA_TX_PIN;
  
  VOR_GPIO_PinMux(UARTA_RX_PORT, UARTA_RX_PIN, UARTA_RX_FUNSEL);  	
  VOR_GPIO_PinMux(UARTA_TX_PORT, UARTA_TX_PIN, UARTA_TX_FUNSEL);
  
  VOR_IOCONFIG->UARTA_RX_PORT[UARTA_RX_PIN] |= IOCONFIG_PORTA_PLEVEL_Msk;
  
  //Watchdog
  WD_BANK.DIR   |=1UL<<WD_PIN;
  WD_BANK.TOGOUT =1UL<<WD_PIN;
  
  VOR_PORTA.DIR |= 1UL<<22;
  VOR_PORTA.DIR |= 1UL<<23;
  VOR_PORTA.SETOUT=1UL<<22; // CSN high
  VOR_PORTA.SETOUT=1UL<<23; // CSN high
  
  // LED
  LED_BANK.DIR |= 1UL<<LED_PIN;
  LED_BANK.CLROUT = 1UL<<LED_PIN;
  
#elif defined(BRD_REB1)
  // REB1 board
  
  //Create default bus condition if FUNSEL moves
  UARTA_TX_BANK.DIR   |=1UL<<UARTA_TX_PIN;
  UARTA_TX_BANK.SETOUT =1UL<<UARTA_TX_PIN;
  
  VOR_GPIO_PinMux(UARTA_RX_PORT, UARTA_RX_PIN, UARTA_RX_FUNSEL);  	
  VOR_GPIO_PinMux(UARTA_TX_PORT, UARTA_TX_PIN, UARTA_TX_FUNSEL);
  
  VOR_IOCONFIG->UARTA_RX_PORT[UARTA_RX_PIN] |= IOCONFIG_PORTA_PLEVEL_Msk;
  
  // set ADC CS_N high
  MAX11619_CS_N_BANK.DIR   |= 1UL<<MAX11619_CS_N_PIN;
  MAX11619_CS_N_BANK.SETOUT = 1UL<<MAX11619_CS_N_PIN;
  
  // LED pins outputs
  LED_D2_BANK.DIR |= 1UL<<LED_D2_PIN;
  LED_D3_BANK.DIR |= 1UL<<LED_D3_PIN;
  LED_D4_BANK.DIR |= 1UL<<LED_D4_PIN;
  
#elif defined(BRD_SIP_EVK)
  // SIP EVK board
  
  // Create default bus condition if FUNSEL moves
  UARTA_TX_BANK.DIR       |=1UL<<UARTA_TX_PIN;
  UARTA_TX_BANK.SETOUT     =1UL<<UARTA_TX_PIN;
  
  UARTB_TX_BANK.DIR       |=1UL<<UARTB_TX_PIN;
  UARTB_TX_BANK.SETOUT     =1UL<<UARTB_TX_PIN;
  
  ROM1_SPIA_CS_BANK.DIR   |=1UL<<ROM1_SPIA_CS_PIN;
  ROM1_SPIA_CS_BANK.SETOUT =1UL<<ROM1_SPIA_CS_PIN;
  
  PMOD_SPIA_CS_BANK.DIR   |=1UL<<PMOD_SPIA_CS_PIN;
  PMOD_SPIA_CS_BANK.SETOUT =1UL<<PMOD_SPIA_CS_PIN;
  
  SPIA_SCK_BANK.DIR       |=1UL<<SPIA_SCK_PIN;
  SPIA_SCK_BANK.CLROUT     =1UL<<SPIA_SCK_PIN;
  
  SPIA_MOSI_BANK.DIR      |=1UL<<SPIA_MOSI_PIN;
  SPIA_MOSI_BANK.CLROUT    =1UL<<SPIA_MOSI_PIN;
  
  ROM1_WP_N_BANK.DIR      |=1UL<<ROM1_WP_N_PIN;
  ROM1_WP_N_BANK.CLROUT    =1UL<<ROM1_WP_N_PIN;
  
  // UART1
  VOR_GPIO_PinMux(UARTB_RX_PORT, UARTB_RX_PIN, UARTB_RX_FUNSEL);  	
  VOR_GPIO_PinMux(UARTB_TX_PORT, UARTB_TX_PIN, UARTB_TX_FUNSEL);
  
  // LEDs
  LED_DS4_PORT->DIR   |= 1UL<<LED_DS4_PIN;
  LED_DS4_PORT->CLROUT = 1UL<<LED_DS4_PIN;
  
  LED_DS5_PORT->DIR   |= 1UL<<LED_DS5_PIN;
  LED_DS5_PORT->CLROUT = 1UL<<LED_DS5_PIN;
  
  // SW2 (switch) GPIO pin needs a pullup
  SW2_PORT->DIR &= ~(1UL<<SW2_PIN);
  VOR_IOCONFIG->SW2_PORT[SW2_PIN] |= IOCONFIG_PORTA_PLEVEL_Msk | (5UL<<IOCONFIG_PORTA_FLTTYPE_Pos);
  
#endif
}
 
 /*******************************************************************************
 **
 ** @brief  Cause a wait for a specified number of milliseconds
 **
 ******************************************************************************/
void waitMs(uint32_t ms)
{
  uint32_t timeout = u32MsCounter + ms;
	while(u32MsCounter != timeout)
  {
	  /* Feed watchdog. */
		WDFeed();
  }
}
 
/*******************************************************************************
 **
 ** @brief  Application entry point.
 **
 ******************************************************************************/
int main(void)
{
	bool enterBootloader = false;
  en_memcheck_t app_valid;
	
  // Enable clocks
 	VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE =
        (CLK_ENABLE_PORTA | CLK_ENABLE_PORTB | CLK_ENABLE_GPIO |
         CLK_ENABLE_IOCONFIG | CLK_ENABLE_PORTIO | CLK_ENABLE_SYSTEM | 
	       CLK_ENABLE_IRQSEL );


  // Configure IO
  ConfigureIO();
  
  // Check bootloader's CRC (and write it if blank)
  Bootloader_CheckOwnCRC();

#ifdef BRD_SIP_EVK
  // Redirect stdio/stderr (printf) to UART1 (connected to FTDI on EVK board)
  DBG_SetStdioOutput(en_stdio_uart1);
#else
  // Redirect stdio/stderr (printf) to UART0 (connected to FTDI on RH-OBC1 board)
  DBG_SetStdioOutput(en_stdio_uart0);
#endif

	UART0Init();
	SecTimerInit();
	MsTimerInit();
	
#ifdef PC_MODE
	printf("\r\nVA108xx Bootloader Version %s\r\n", SOFTWARE_VERSION_STR);
	printf("EF_ID = 0x%08x\r\n" , VOR_SYSCONFIG->EF_ID);
  printf("RST_STAT = 0x%08x\r\n" , VOR_SYSCONFIG->RST_STAT);
	printf("Press spacebar to enter bootloader command");
#endif

  // for timeout period, looks for heralding character
	u32MsCounter = 0;
	while(u32MsCounter < BOOTLOADER_MSEC_TIMEOUT){
	  if (UART0IsRxByteAvailable()){
			if(UART0RxByte() == BOOTLOADER_HERALDING_CHAR){
				// caught enter bootloader cmd
				enterBootloader = true;
#ifndef PC_MODE
			  // send response to heralding char
				UART0TxByte(BOOTLOADER_HERALDING_RESP_CHAR);
#endif
				break;
			}
	  }
		waitMs(10);
		
#ifdef PC_MODE		
		if ((u32MsCounter % 1000) == 0){
			printf(".");
		}
#endif
		
		// Feed watchdog
		WDFeed();
  }
	
	if(false == enterBootloader){
		// no hearalding char heard - attempt to run application
    app_valid = Bootloader_CheckAppIsValid();
		if((app_valid & en_memcheck_app_a_valid) > 0){
#ifdef PC_MODE
      // print info
			printf("\r\nApp A valid. ");
      if ((app_valid & en_memcheck_app_b_valid) > 0){
        printf("App B valid. ");
      }else{
        printf("App B invalid. ");
      }
      printf("Running App A\r\n");
			waitMs(10);
#endif
			Bootloader_RunApp(en_runapp_a);
		}
		else if ((app_valid & en_memcheck_app_b_valid) > 0){
#ifdef PC_MODE
      // print info
			printf("\r\nApp A invalid. App B valid. Running App B\r\n");
			waitMs(10);
#endif
			Bootloader_RunApp(en_runapp_b);
    }else{
#ifdef PC_MODE
			printf("\r\nBoth app images invalid. Upload new code to app A or B\r\n");
#endif
		}
	}
	
	// bootloader loop - wait for code upload
  while(1)
  {		
		if(u32SecToggle)
		{
			u32SecToggle = 0;  
#ifdef BRD_SIP_EVK
      LED_DS5_PORT->TOGOUT = 1UL<<LED_DS5_PIN;  // Toggle LED DS5 on SIP EVK
#elif defined(BRD_RH_OBC1)
      LED_BANK.TOGOUT=1UL<<LED_PIN;
#endif
		}
		
		Bootloader_Task();
		
		// Feed watchdog
		WDFeed();
  }
}   // main

/*******************************************************************************
 **
 ** @brief  The ISR for tracking time.
 **
 ** Counts time in number of seconds.
 **
 ******************************************************************************/
void OC27_IRQHandler(void)
{
  ++u32SecondsCounter;
	u32SecToggle=1;
}

void OC28_IRQHandler(void)
{
  ++u32MsCounter;
	u32MsToggle=1;
}

/*****************************************************************************/ 
/* End of file                                                               */ 
/*****************************************************************************/
